import argparse
import os
import urllib.request


URL = 'https://github.com/avidale/python-ruwordnet/releases/download/0.0.2/ruwordnet.db'


def main():
    parser = argparse.ArgumentParser(description='Tools for RuWordNet')
    subparsers = parser.add_subparsers(dest='subparser')
    download = subparsers.add_parser('download', help='Download the model')
    download.add_argument('-u', '--url', default=URL, help='url of the model to download')
    args = parser.parse_args()
    if args.subparser == 'download':
        dirname = os.path.join(os.path.dirname(__file__), 'static')
        if not os.path.exists(dirname):
            os.makedirs(dirname, exist_ok=True)
        destination = os.path.join(dirname, 'ruwordnet.db')
        print('downloading a ruwordnet model from', args.url)
        urllib.request.urlretrieve(args.url, destination)


if __name__ == '__main__':
    main()



from typing import List

from sqlalchemy import Column, String, ForeignKey, Table
from sqlalchemy.orm import relationship
from sqlalchemy.ext.declarative import declarative_base

Base = declarative_base()


composition_table = Table(
    'composition_relation',
    Base.metadata,
    Column('word_id', String(), ForeignKey("sense.id"), primary_key=True),
    Column('phrase_id', String(), ForeignKey("sense.id"), primary_key=True)
)


derivation_table = Table(
    'derivation_relation',
    Base.metadata,
    Column('source_id', String(), ForeignKey("sense.id"), primary_key=True),
    Column('derivative_id', String(), ForeignKey("sense.id"), primary_key=True)
)


class Sense(Base):
    __tablename__ = 'sense'
    metadata = Base.metadata
    id: str = Column(String(), primary_key=True, index=True)
    name: str = Column(String(), index=True)
    lemma: str = Column(String(), index=True)
    # todo: add more fields
    """
    main_word=None,
    synt_type=None,
    poses=None,
    meaning=None,
    part_of_speech=None,
    concept_id=None,
    entry_id=None,
    """

    synset_id: str = Column(String(), ForeignKey('synset.id'))
    synset: 'Synset' = relationship("Synset", back_populates="senses")

    words: List['Sense'] = relationship(
        "Sense",
        secondary=composition_table,
        back_populates='phrases',
        primaryjoin=id == composition_table.c.phrase_id,
        secondaryjoin=id == composition_table.c.word_id,
    )
    phrases: List['Sense'] = relationship(
        "Sense",
        secondary=composition_table,
        back_populates='words',
        primaryjoin=id == composition_table.c.word_id,
        secondaryjoin=id == composition_table.c.phrase_id,
    )

    sources: List['Sense'] = relationship(
        "Sense",
        secondary=derivation_table,
        back_populates='derivations',
        primaryjoin=id == derivation_table.c.derivative_id,
        secondaryjoin=id == derivation_table.c.source_id,
    )
    derivations: List['Sense'] = relationship(
        "Sense",
        secondary=derivation_table,
        back_populates='sources',
        primaryjoin=id == derivation_table.c.source_id,
        secondaryjoin=id == derivation_table.c.derivative_id,
    )

    def __repr__(self):
        return 'Sense(id="{}", name="{}")'.format(self.id, self.name)


hypernymy_table = Table(
    'hypernym_relation',
    Base.metadata,
    Column('hyponym_id', String(), ForeignKey("synset.id"), primary_key=True),
    Column('hypernym_id', String(), ForeignKey("synset.id"), primary_key=True)
)


domains_table = Table(
    'domain_relation',
    Base.metadata,
    Column('domain_id', String(), ForeignKey("synset.id"), primary_key=True),
    Column('domain_item_id', String(), ForeignKey("synset.id"), primary_key=True)
)


meronymy_table = Table(
    'meronymy_relation',
    Base.metadata,
    Column('meronym_id', String(), ForeignKey("synset.id"), primary_key=True),
    Column('holonym_id', String(), ForeignKey("synset.id"), primary_key=True)
)


instances_table = Table(
    'instance_relation',
    Base.metadata,
    Column('instance_id', String(), ForeignKey("synset.id"), primary_key=True),
    Column('class_id', String(), ForeignKey("synset.id"), primary_key=True)
)


entailment_table = Table(
    'entailment_relation',
    Base.metadata,
    Column('premise_id', String(), ForeignKey("synset.id"), primary_key=True),
    Column('conclusion_id', String(), ForeignKey("synset.id"), primary_key=True)
)


cause_table = Table(
    'cause_relation',
    Base.metadata,
    Column('cause_id', String(), ForeignKey("synset.id"), primary_key=True),
    Column('effect_id', String(), ForeignKey("synset.id"), primary_key=True)
)


pos_synonymy_table = Table(
    'pos_synonymy_relation',
    Base.metadata,
    Column('left_id', String(), ForeignKey("synset.id"), primary_key=True),
    Column('right_id', String(), ForeignKey("synset.id"), primary_key=True)
)


antonymy_table = Table(
    'antonymy_relation',
    Base.metadata,
    Column('left_id', String(), ForeignKey("synset.id"), primary_key=True),
    Column('right_id', String(), ForeignKey("synset.id"), primary_key=True)
)


class Synset(Base):
    __tablename__ = 'synset'
    metadata = Base.metadata
    id: str = Column(String(), primary_key=True, index=True)
    title: str = Column(String())
    definition: str = Column(String())
    part_of_speech: str = Column(String())

    senses: List[Sense] = relationship("Sense", order_by=Sense.id, back_populates="synset")

    hypernyms: List['Synset'] = relationship(
        "Synset",
        secondary=hypernymy_table,
        back_populates='hyponyms',
        primaryjoin=id == hypernymy_table.c.hyponym_id,
        secondaryjoin=id == hypernymy_table.c.hypernym_id,
    )
    hyponyms: List['Synset'] = relationship(
        "Synset",
        secondary=hypernymy_table,
        back_populates='hypernyms',
        primaryjoin=id == hypernymy_table.c.hypernym_id,
        secondaryjoin=id == hypernymy_table.c.hyponym_id,
    )

    domains = relationship(
        "Synset",
        secondary=domains_table,
        back_populates='domain_items',
        primaryjoin=id == domains_table.c.domain_item_id,
        secondaryjoin=id == domains_table.c.domain_id,
    )
    domain_items: List['Synset'] = relationship(
        "Synset",
        secondary=domains_table,
        back_populates='domains',
        primaryjoin=id == domains_table.c.domain_id,
        secondaryjoin=id == domains_table.c.domain_item_id,
    )

    meronyms: List['Synset'] = relationship(
        "Synset",
        secondary=meronymy_table,
        back_populates='holonyms',
        primaryjoin=id == meronymy_table.c.holonym_id,
        secondaryjoin=id == meronymy_table.c.meronym_id,
    )
    holonyms: List['Synset'] = relationship(
        "Synset",
        secondary=meronymy_table,
        back_populates='meronyms',
        primaryjoin=id == meronymy_table.c.meronym_id,
        secondaryjoin=id == meronymy_table.c.holonym_id,
    )

    instances: List['Synset'] = relationship(
        "Synset",
        secondary=instances_table,
        back_populates='classes',
        primaryjoin=id == instances_table.c.class_id,
        secondaryjoin=id == instances_table.c.instance_id,
    )
    classes: List['Synset'] = relationship(
        "Synset",
        secondary=instances_table,
        back_populates='instances',
        primaryjoin=id == instances_table.c.instance_id,
        secondaryjoin=id == instances_table.c.class_id,
    )

    premises: List['Synset'] = relationship(
        "Synset",
        secondary=entailment_table,
        back_populates='conclusions',
        primaryjoin=id == entailment_table.c.conclusion_id,
        secondaryjoin=id == entailment_table.c.premise_id,
    )
    conclusions: List['Synset'] = relationship(
        "Synset",
        secondary=entailment_table,
        back_populates='premises',
        primaryjoin=id == entailment_table.c.premise_id,
        secondaryjoin=id == entailment_table.c.conclusion_id,
    )

    causes: List['Synset'] = relationship(
        "Synset",
        secondary=cause_table,
        back_populates='effects',
        primaryjoin=id == cause_table.c.effect_id,
        secondaryjoin=id == cause_table.c.cause_id,
    )
    effects: List['Synset'] = relationship(
        "Synset",
        secondary=cause_table,
        back_populates='causes',
        primaryjoin=id == cause_table.c.cause_id,
        secondaryjoin=id == cause_table.c.effect_id,
    )

    # pos_synonyms and antonyms are duplicated - it is easier than dirty SQLAlchemy hacks
    pos_synonyms: List['Synset'] = relationship(
        "Synset",
        secondary=pos_synonymy_table,
        back_populates='pos_synonyms_reverse',
        primaryjoin=id == pos_synonymy_table.c.right_id,
        secondaryjoin=id == pos_synonymy_table.c.left_id,
    )
    pos_synonyms_reverse: List['Synset'] = relationship(
        "Synset",
        secondary=pos_synonymy_table,
        back_populates='pos_synonyms',
        primaryjoin=id == pos_synonymy_table.c.left_id,
        secondaryjoin=id == pos_synonymy_table.c.right_id,
    )

    antonyms: List['Synset'] = relationship(
        "Synset",
        secondary=antonymy_table,
        back_populates='antonyms_reverse',
        primaryjoin=id == antonymy_table.c.right_id,
        secondaryjoin=id == antonymy_table.c.left_id,
    )
    antonyms_reverse: List['Synset'] = relationship(
        "Synset",
        secondary=antonymy_table,
        back_populates='antonyms',
        primaryjoin=id == antonymy_table.c.left_id,
        secondaryjoin=id == antonymy_table.c.right_id,
    )

    def __repr__(self):
        return 'Synset(id="{}", title="{}")'.format(self.id, self.title)



import os

from sqlalchemy import create_engine
from sqlalchemy.orm import sessionmaker

from .models import Base


def get_default_session(filename=None):
    if filename is None:
        filename = os.path.join(
            os.path.dirname(__file__),
            'static',
            'ruwordnet.db'
        )
    if not os.path.exists(filename):
        raise FileNotFoundError(
            f'The file {filename} was not found. '
            f'Please make sure you have provided a correct database filename.'
        )

    engine = create_engine(f'sqlite:///{filename}', echo=False)
    Base.metadata.create_all(engine)

    Session = sessionmaker()
    Session.configure(bind=engine)
    session = Session()
    return session



import os
import setuptools

description = "A Python wrapper for the RuWordNet thesaurus."
long_description = description
if os.path.exists("README.md"):
    with open("README.md", "r", encoding="utf-8") as fh:
        long_description = fh.read()


setuptools.setup(
    name="ruwordnet",
    version="0.0.3",
    author="David Dale",
    author_email="dale.david@mail.ru",
    description=description,
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/avidale/python-ruwordnet",
    packages=setuptools.find_packages(),
    license="MIT",
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    install_requires=['sqlalchemy'],
    entry_points={
            "console_scripts": [
                "ruwordnet=ruwordnet.__main__:main",
        ]
    }
)



from typing import List, Union
from .models import Sense, Synset
from .utils import get_default_session


class RuWordNet:
    def __init__(self, filename_or_session=None):
        if filename_or_session is None:
            session = get_default_session()
        elif isinstance(filename_or_session, str):
            session = get_default_session(filename_or_session)
        else:
            session = filename_or_session
        self.session = session

    @property
    def synsets(self) -> List[Sense]:
        """ List of all available synsets """
        return self.session.query(Synset).all()

    @property
    def senses(self) -> List[Sense]:
        """ List of all available senses """
        return self.session.query(Sense).all()

    def __getitem__(self, item: str) -> Union[Synset, Sense, List[Sense]]:
        """ Retrieve sense or synset by its id """
        synset = self.session.query(Synset).filter_by(id=item).first()
        if synset:
            return synset
        sense = self.session.query(Sense).filter_by(id=item).first()
        if sense:
            return sense

        senses = self.get_senses(item)
        if senses:
            return senses
        raise KeyError()

    def get_senses(self, lemma: str) -> List[Sense]:
        """ Retrieve a list of senses by a given lemma """
        q = lemma.upper().strip()
        return self.session.query(Sense).filter_by(lemma=q).all()

    def get_synsets(self, lemma: str) -> List[Synset]:
        """ Retrieve a list of synsets by a given lemma """
        return [sense.synset for sense in self.get_senses(lemma) if sense.synset]
